package pay.platform.api.system.event;

import pay.platform.api.system.cache.PlatformProperties;
import pay.platform.api.system.cache.RouteCache;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.token.UserCacheProvider;
import jakarta.annotation.Nonnull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;

/**
 * 刷新在线用户路由监听器
 */
@Component
@RequiredArgsConstructor
@Slf4j
@Async
public class RefreshOnlineUserRouteListener implements ApplicationListener<RefreshOnlineUserRouteEvent> {

    private final UserCacheProvider userCacheProvider;

    private final PlatformProperties properties;

    private final RouteCache routeCache;

    @Override
    public void onApplicationEvent(@Nonnull RefreshOnlineUserRouteEvent event) {
        log.info("【刷新在线用户路由事件】刷新在线用户路由信息>>>>>>>>>>>开始");
        Boolean refreshRoutes = properties.getRefreshRoutes();
        if (!refreshRoutes) {
            log.warn("【刷新在线用户路由事件】刷新在线用户路由信息>>>>>>>>>>>未开启");
            return;
        }
        Optional<List<UserInfo>> users = userCacheProvider.getCacheUsers();
        if (users.isPresent()) {
            List<UserInfo> userInfoList = users.get();
            for (UserInfo userInfo : userInfoList) {
                log.info("【刷新在线用户路由事件】开始清理登陆用户缓存信息: {}", userInfo.getUsername());
                // TODO 是否刷新用户权限信息
                userCacheProvider.clearUser(userInfo.getUsername());
                log.info("【刷新在线用户路由事件】开始清理登陆用户路由信息: {}", userInfo.getUsername());
                routeCache.removeCache(userInfo.getUserid());
            }
        }
        log.info("【刷新在线用户路由事件】刷新在线用户路由信息>>>>>>>>>>>结束");
    }
}
