package pay.platform.api.system.servcie.impl;


import cn.hutool.core.bean.BeanUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryColumn;
import com.mybatisflex.core.query.QueryMethods;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.StringUtil;
import org.springframework.stereotype.Service;
import pay.platform.api.system.model.query.PayOrderQuery;
import pay.platform.api.system.model.vo.*;
import pay.platform.api.system.model.vo.HomeVO.TotalVO;
import pay.platform.api.system.servcie.PayOrderService;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.domain.PayOrder;
import pay.platform.domain.table.PayOrderTableDef;
import pay.platform.mapper.PayOrderMapper;

import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 通道管理 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayOrderServiceImpl extends BaseService<PayOrderMapper, PayOrder> implements PayOrderService {


    @Override
    public BasePage<PayOrderVO> queryPage(PayOrderQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where("1 = 1")
                .and(PayOrderTableDef.PAY_ORDER.ORDER_NO.like(query.getOrderNo(), StringUtil.isNotBlank(query.getOrderNo())))
                .and(PayOrderTableDef.PAY_ORDER.PLATFORM_ORDER_NO.like(query.getPlatformOrderNo(), StringUtil.isNotBlank(query.getPlatformOrderNo())))
                .and(PayOrderTableDef.PAY_ORDER.MERCHANT_ID.like(query.getMerchantId(), query.getMerchantId() != null)
                        .and(PayOrderTableDef.PAY_ORDER.AGENT_ID.like(query.getAgentId(), query.getAgentId() != null)
                                .and(PayOrderTableDef.PAY_ORDER.STATUS.eq(query.getStatus(), query.getStatus() != null)))).orderBy(PayOrderTableDef.PAY_ORDER.CREATE_TIME.desc());
        Page<PayOrder> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<PayOrderVO> vos = BeanUtil.copyToList(page.getRecords(), PayOrderVO.class);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
    }

    public QueryWrapper getQueryWrapper(Calendar calendar, int period, QueryColumn queryColumn) {
        //当天支付
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where("1 = 1")
                .select(PayOrderTableDef.PAY_ORDER.STATUS, QueryMethods.sum(PayOrderTableDef.PAY_ORDER.AMOUNT).as("amount"),
                        QueryMethods.sum(PayOrderTableDef.PAY_ORDER.PLATFORM_RATE_AMOUNT).as("platformRateAmount"),
                        QueryMethods.count(PayOrderTableDef.PAY_ORDER.STATUS).as("day")
                )
                .and(PayOrderTableDef.PAY_ORDER.YEAR.eq(calendar.get(Calendar.YEAR)))
//                .and(PayOrderTableDef.PAY_ORDER.DAY.eq(calendar.get(period)))
                .and(queryColumn.eq(calendar.get(period)))
                .groupBy(PayOrderTableDef.PAY_ORDER.STATUS);
        return queryWrapper;
    }

    @Override
    public HomeVO home() {
        HomeVO homeVO = new HomeVO();
        Calendar calendar = Calendar.getInstance();
        //当天支付
        var result = getResult(getQueryWrapper(calendar, Calendar.DAY_OF_YEAR, PayOrderTableDef.PAY_ORDER.DAY));

        homeVO.setToday(new TotalVO(new QuantityVO(result.payCount(), result.paySuccessCount()), new AmountVO(result.payAmount(), result.paySuccessAmount()), new EarningsVO(result.dayFee(), result.dayFee())));

        //前一天支付
        calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_YEAR, calendar.get(Calendar.DAY_OF_YEAR) - 1);
        result = getResult(getQueryWrapper(calendar, Calendar.DAY_OF_YEAR, PayOrderTableDef.PAY_ORDER.DAY));
        homeVO.setYesterday(new TotalVO(new QuantityVO(result.payCount(), result.paySuccessCount()), new AmountVO(result.payAmount(), result.paySuccessAmount()), new EarningsVO(result.dayFee(), result.dayFee())));

        //周支付
        calendar = Calendar.getInstance();
        result = getResult(getQueryWrapper(calendar, Calendar.WEEK_OF_YEAR, PayOrderTableDef.PAY_ORDER.WEEK));
        homeVO.setLastWeek(new TotalVO(new QuantityVO(result.payCount(), result.paySuccessCount()), new AmountVO(result.payAmount(), result.paySuccessAmount()), new EarningsVO(result.dayFee(), result.dayFee())));

        //月支付
        calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH, calendar.get(Calendar.MONTH) + 1);
        result = getResult(getQueryWrapper(calendar, Calendar.MONTH, PayOrderTableDef.PAY_ORDER.MONTH));
        homeVO.setLastMonth(new TotalVO(new QuantityVO(result.payCount(), result.paySuccessCount()), new AmountVO(result.payAmount(), result.paySuccessAmount()), new EarningsVO(result.dayFee(), result.dayFee())));

        return homeVO;
    }

    private Result getResult(QueryWrapper queryWrapper) {
        var payOrders = mapper.selectListByQuery(queryWrapper);
        //统计当天支付订单总数
        var payCount = payOrders.stream().collect(Collectors.summarizingDouble(PayOrder::getDay)).getSum();
        //统计当天成功支付订单总数
        var paySuccessCount = payOrders.stream().filter(item -> item.getStatus() == 1).collect(Collectors.summarizingDouble(PayOrder::getDay)).getSum();
        //统计支付金额
        var payAmount = payOrders.stream().collect(Collectors.summarizingDouble(payOrder -> payOrder.getAmount().doubleValue())).getSum();
        var paySuccessAmount = payOrders.stream().filter(item -> item.getStatus() == 1).collect(Collectors.summarizingDouble(payOrder -> payOrder.getAmount().doubleValue())).getSum();
        DecimalFormat decimalFormat = new DecimalFormat("0.00");
        payAmount = Double.valueOf(decimalFormat.format(payAmount));
        paySuccessAmount = Double.valueOf(decimalFormat.format(paySuccessAmount));
        //统计当天成功支付订单平台费率金额
        var dayFee = payOrders.stream().filter(item -> item.getStatus() == 1).collect(Collectors.summarizingDouble(payOrder -> payOrder.getPlatformRateAmount().doubleValue())).getSum();
        dayFee = Double.valueOf(decimalFormat.format(dayFee));
        Result result = new Result(payCount, paySuccessCount, payAmount, paySuccessAmount, dayFee);
        return result;
    }

    private record Result(double payCount, double paySuccessCount, double payAmount, double paySuccessAmount,
                          double dayFee) {
    }
}