package pay.platform.core.security.filter;

import cn.hutool.core.util.StrUtil;
import pay.platform.core.security.model.OnlineUser;
import pay.platform.core.security.service.UserDetailServiceImpl;
import pay.platform.core.security.token.JwtTokenRedisCacheProvider;
import pay.platform.core.security.token.UserCacheProvider;
import pay.platform.core.security.util.JwtUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Optional;

/**
 * token认证过滤器
 */
@Slf4j
@RequiredArgsConstructor
@Component
public class JwtTokenAuthenticationFilter extends OncePerRequestFilter {
    private final JwtTokenRedisCacheProvider jwtTokenRedisCacheProvider;
    private final UserDetailServiceImpl userDetailService;
    @Lazy
    @Resource
    private UserCacheProvider userCacheProvider;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String _jwtToken = JwtUtil.getTokenByRequest(request);
        if (StrUtil.isNotBlank(_jwtToken)) {
            Optional<OnlineUser> online = jwtTokenRedisCacheProvider.getOnlineFromToken(_jwtToken);
            if (online.isPresent()) {
                if (SecurityContextHolder.getContext().getAuthentication() == null) {
                    UserDetails userDetails = userDetailService.loadUserByUsername(JwtUtil.getUsername(_jwtToken));

                    UsernamePasswordAuthenticationToken authenticationToken =
                            new UsernamePasswordAuthenticationToken(
                                    userDetails.getUsername(),
                                    userDetails,
                                    userDetails.getAuthorities()
                            );
                    authenticationToken.setDetails(
                            new WebAuthenticationDetailsSource().buildDetails(request)
                    );
                    SecurityContextHolder.getContext().setAuthentication(authenticationToken);
                }
            } else {
                try {
                    String username = JwtUtil.getUsername(_jwtToken);
                    Optional.ofNullable(username).ifPresent((u) -> userCacheProvider.clearUser(u));
                } catch (Exception ignored) {
                }
            }
            jwtTokenRedisCacheProvider.checkRenewal(_jwtToken);
        }
        filterChain.doFilter(request, response);
    }
}
