package pay.platform.core.security.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import pay.platform.api.system.servcie.PayAgentService;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.token.RedisCacheUserProvider;
import pay.platform.domain.PayAgent;

import java.util.List;
import java.util.Optional;

/**
 * 用户详细信息服务
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class UserDetailServiceImpl implements UserDetailsService {

    private final RedisCacheUserProvider redisCacheUserProvider;

    private final PayAgentService sysUserService;


    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        log.info("获取授权用户信息: username: {}", username);
        Optional<UserInfo> optional = redisCacheUserProvider.getUser(username);
        if (optional.isPresent()) {
            UserInfo userInfo = optional.get();
            log.info("获取授权用户信息->缓存获取 : {}", userInfo);
            return userInfo;
        }
        log.info("获取授权用户信息->重新从库获取");
        // 测试用例
//        UserInfo info = new UserInfo();
//        info.setPassword(PasswordUtil.encoder("123456"));
//        info.setUsername("admin");
//        info.setRoles(Arrays.asList("ROLE_ADMIN"));
//        info.setPermissions(Arrays.asList("test::query", "test:add"));
//        info.setIsEnabled(1);
        // 正式
        Optional<PayAgent> userOptional = sysUserService.loadUserByUsername(username);
        PayAgent user = userOptional.orElseThrow(() -> new UsernameNotFoundException("根据用户名未找到用户信息"));
        UserInfo userInfo = UserInfo.convert(
                user,
                List.of(),
                List.of());
        redisCacheUserProvider.cacheUser(username, userInfo);
        return userInfo;
    }
}
