package pay.platform.core.security.token;

import jakarta.annotation.Nonnull;
import jakarta.annotation.Nullable;
import jakarta.servlet.http.HttpServletRequest;
import pay.platform.core.security.model.OnlineUser;
import pay.platform.core.security.model.UserInfo;

import java.util.Optional;

/**
 * 令牌提供程序
 */
public interface TokenProvider {
    /**
     * 创建token
     *
     * @param userInfo .
     * @return token
     */
    String createToken(@Nonnull UserInfo userInfo);

    /**
     * 创建token
     *
     * @param userInfo .
     * @param request  .
     * @return token
     */
    String createToken(@Nonnull UserInfo userInfo, @Nullable HttpServletRequest request);

    /**
     * 获取在线用户
     *
     * @param token .
     * @return .
     */
    Optional<OnlineUser> getOnlineFromToken(@Nonnull String token);

    /**
     * 是否延长token时间
     *
     * @param token .
     * @return .
     */
    boolean checkRenewal(@Nonnull String token);

    /**
     * 移除token
     *
     * @param token .
     * @return .
     */
    boolean removeToken(String token);
}
