package pay.platform.api.system.controller;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.mybatisflex.core.query.QueryWrapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pay.platform.api.system.model.query.PayOrderQuery;
import pay.platform.api.system.model.request.PayRequest;
import pay.platform.api.system.model.request.TestRequest;
import pay.platform.api.system.model.vo.PayOrderVO;
import pay.platform.api.system.servcie.PayMerchantAppKeyService;
import pay.platform.api.system.servcie.PayOrderService;
import pay.platform.api.system.util.SignUtil;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.PayOrder;
import pay.platform.domain.table.PayOrderTableDef;

import java.io.Serializable;
import java.util.List;
import java.util.TreeMap;

/**
 * 订单 控制层。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@RestController
@RequestMapping("/merchant/pay/order")
@Tag(name = "订单控制层")
public class PayOrderController {

    @Autowired
    private PayOrderService PayOrderService;
    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;

    /**
     * 添加 订单
     *
     * @param PayOrder 订单
     * @return Result.OK({ @ code true } 添加成功 ， { @ code false } 添加失败
     */
    @PostMapping("/save")
    @Operation(summary = "添加订单")
    public Result<Boolean> save(@RequestBody PayOrder PayOrder) {
        PayOrder.setMerchantId(SecurityUtil.getCurrentUser().getUserid());
        return Result.OK(PayOrderService.save(PayOrder));
    }

    /**
     * 添加 订单
     *
     * @param 订单
     * @return Result.OK({ @ code true } 添加成功 ， { @ code false } 添加失败
     */
    @PostMapping("/test")
    @Operation(summary = "测试支付订单")
    public Result<JSONObject> test(@RequestBody TestRequest testRequest) {
        var currentUser = SecurityUtil.getCurrentUser();
        var payMerchant = payMerchantAppKeyService.info(currentUser.getUserid());

        PayRequest payRequest = new PayRequest();
        payRequest.setBody("测试下单");
        payRequest.setOrderNo(testRequest.getBizOrderNo());
        payRequest.setCurrentTime(System.currentTimeMillis());
        payRequest.setAppId(payMerchant.getAppId());
        payRequest.setSubject(testRequest.getTitle());
        payRequest.setChannelId(testRequest.getChannelId());
        payRequest.setAmount(testRequest.getTotalMoney());
        payRequest.setNotifyUrl("https://open.gagapay.pro/api/return");
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("orderNo", payRequest.getOrderNo());
        treeMap.put("amount", payRequest.getAmount().toString());
        treeMap.put("subject", payRequest.getSubject());
        treeMap.put("body", payRequest.getBody());
        treeMap.put("currentTime", payRequest.getCurrentTime().toString());
        treeMap.put("notifyUrl", payRequest.getNotifyUrl());
        treeMap.put("appId", payMerchant.getAppId());
        payRequest.setSign(SignUtil.getSign(payMerchant.getAppKey(), treeMap));
        var post = HttpUtil.post("https://open.gagapay.pro/api/pay/alipay/test", JSONObject.toJSONString(payRequest));
        return Result.OK(JSONObject.parseObject(post));
    }

    /**
     * 根据主键删除订单
     *
     * @param id 主键
     * @return Result.OK({ @ code true } 删除成功 ， { @ code false } 删除失败
     */
    @DeleteMapping("/remove/{id}")
    @Operation(summary = "根据主键删除订单")
    @Parameters(value = {
            @Parameter(name = "id", description = "id", required = true)
    })
    public Result<Boolean> remove(@PathVariable Serializable id) {
        return Result.OK(PayOrderService.removeById(id));
    }


    /**
     * 根据主键更新订单
     *
     * @param PayOrder 订单
     * @return Result.OK({ @ code true } 更新成功 ， { @ code false } 更新失败
     */
    @PutMapping("/update")
    @Operation(summary = "根据主键更新订单")
    public Result<Boolean> update(@RequestBody PayOrder PayOrder) {
        return Result.OK(PayOrderService.updateById(PayOrder));
    }


    /**
     * 查询所有订单
     *
     * @return Result.OK(所有数据
     */
    @GetMapping("/list")
    @Operation(summary = "查询所有订单")
    public Result<List<PayOrder>> list() {
        QueryWrapper queryWrapper = new QueryWrapper().eq(String.valueOf(PayOrderTableDef.PAY_ORDER.MERCHANT_ID), SecurityUtil.getCurrentUser().getUserid());

        return Result.OK(PayOrderService.list(queryWrapper));
    }


    /**
     * 根据订单主键获取详细信息。
     *
     * @param id PayOrder主键
     * @return Result.OK(订单详情
     */
    @GetMapping("/getInfo/{id}")
    @Operation(summary = "根据订单主键获取详细信息")
    @Parameters(value = {
            @Parameter(name = "id", description = "id", required = true)
    })
    public Result<PayOrder> getInfo(@PathVariable Serializable id) {
        return Result.OK(PayOrderService.getById(id));
    }


    /**
     * 分页查询订单
     *
     * @param query 分页对象
     * @return Result.OK(分页对象
     */
    @GetMapping("/page")
    @Operation(summary = "分页查询订单")
    public Result<BasePage<PayOrderVO>> page(@ParameterObject PayOrderQuery query) {
        return Result.OK(PayOrderService.queryPage(query));
    }

    /**
     * 订单回调
     *
     * @param query 分页对象
     * @return Result.OK(分页对象
     */
    @GetMapping("/callback/{id}")
    @Operation(summary = "订单回调")
    public Result<Boolean> callback(@PathVariable("id") String id) {
        PayOrderService.notifyMerchant(id);
        return Result.OK();
    }

    /**
     * 订单状态查询
     *
     * @param query 分页对象
     * @return Result.OK(分页对象
     */
    @GetMapping("/status/{id}")
    @Operation(summary = "订单状态查询")
    public Result<String> syncOrderStatus(@PathVariable("id") String id) {
        return PayOrderService.syncOrderStatus(id);
    }


}