package pay.platform.api.system.servcie.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.alipay.api.domain.AlipayTradeQueryModel;
import com.alipay.api.response.AlipayTradeQueryResponse;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.CollectionUtil;
import com.mybatisflex.core.util.StringUtil;
import jakarta.validation.constraints.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pay.platform.alipay.enums.AliPayApi;
import pay.platform.alipay.enums.AliPayApiConfig;
import pay.platform.api.system.model.query.PayOrderQuery;
import pay.platform.api.system.model.response.OrderResponse;
import pay.platform.api.system.model.vo.PayOrderVO;
import pay.platform.api.system.servcie.PayChannelService;
import pay.platform.api.system.servcie.PayMerchantAppKeyService;
import pay.platform.api.system.servcie.PayOrderService;
import pay.platform.api.system.servcie.PayPlatformSettingService;
import pay.platform.api.system.util.SignUtil;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.PayMerchantAppKey;
import pay.platform.domain.PayOrder;
import pay.platform.domain.table.PayOrderTableDef;
import pay.platform.mapper.PayOrderMapper;

import java.util.List;
import java.util.TreeMap;

/**
 * 通道管理 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayOrderServiceImpl extends BaseService<PayOrderMapper, PayOrder> implements PayOrderService {

    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;
    @Autowired
    private PayChannelService payChannelService;
    @Autowired
    private PayPlatformSettingService payPlatformSettingService;
    @Override
    public BasePage<PayOrderVO> queryPage(PayOrderQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where("1 = 1")
                .and(PayOrderTableDef.PAY_ORDER.ORDER_NO.like(query.getOrderNo(), StringUtil.isNotBlank(query.getOrderNo())))
                .and(PayOrderTableDef.PAY_ORDER.PLATFORM_ORDER_NO.like(query.getPlatformOrderNo(), StringUtil.isNotBlank(query.getPlatformOrderNo())))
                .and(PayOrderTableDef.PAY_ORDER.MERCHANT_ID.eq(SecurityUtil.getCurrentUser().getUserid(), StringUtil.isNotBlank(SecurityUtil.getCurrentUser().getUserid()))
                        .and(PayOrderTableDef.PAY_ORDER.STATUS.eq(query.getStatus(), query.getStatus() != null))).orderBy(PayOrderTableDef.PAY_ORDER.CREATE_TIME.desc());
        Page<PayOrder> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<PayOrderVO> vos = BeanUtil.copyToList(page.getRecords(), PayOrderVO.class);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
    }


    /**
     * 回调商户
     *
     * @param id
     */
    @Override
    public void notifyMerchant(String id) {
        var payOrder = getById(id);
        var payMerchantAppKey = payMerchantAppKeyService.getByMerchantId(payOrder.getMerchantId());
        var orderResponse = getOrderResponse(payOrder, payMerchantAppKey);
        var post = HttpUtil.post(payOrder.getNotifyUrl(), JSONObject.toJSONString(orderResponse));
        if (post.contains("success")) {
            payOrder.setNotifyCount(1);
            payOrder.setNotifyStatus(1);
            updateById(payOrder);
        } else {
            payOrder.setNotifyCount(1);
            updateById(payOrder);
        }
    }

    @Override
    public Result<String> syncOrderStatus(String id) {

        try {
            var payOrder = getById(id);
            var payChannel = payChannelService.getById(payOrder.getChannelId());
            var aliPayApiConfig = AliPayApiConfig.builder();
            if (payChannel.getType() == 2) {
                aliPayApiConfig = AliPayApiConfig.builder()
                        .setAppId(payChannel.getAppId())
                        .setAliPayPublicKey(payChannel.getPublicKey())
                        .setPrivateKey(payChannel.getPrivateKey())
                        .setServiceUrl(AliPayApiConfig.SANDBOX_GATEWAY_URL)
                        .build();
            } else {
                var payPlatformSettings = payPlatformSettingService.list();
                if (CollectionUtil.isNotEmpty(payPlatformSettings)) {
                    var payPlatformSetting = payPlatformSettings.get(0);
                    aliPayApiConfig = AliPayApiConfig.builder()
                            .setAppId(payPlatformSetting.getAppId())
                            .setAliPayPublicKey(payPlatformSetting.getPublicKey())
                            .setPrivateKey(payPlatformSetting.getPrivateKey())
                            .setServiceUrl(AliPayApiConfig.SANDBOX_GATEWAY_URL)
                            .build();
                }
            }
            // 商户订单号
            String outTradeNo = payOrder.getPlatformOrderNo();

            AlipayTradeQueryModel model = new AlipayTradeQueryModel();
            model.setOutTradeNo(outTradeNo);
            AlipayTradeQueryResponse response = AliPayApi.tradeQueryToResponse(aliPayApiConfig.getAliPayClient(), model, null);
            System.out.println(response.getBody());
            if (response.isSuccess()) {
                if (response.getTradeStatus().equals("TRADE_SUCCESS")) {
                    payOrder.setStatus(1);
                    updateById(payOrder);
                    notifyMerchant(id);
                }
            } else {
                return Result.NG(response.getSubMsg());
            }
//            log.info(STR."response=\n\{JSONUtil.formatJsonStr(JSONUtil.toJsonStr(response))}");

        } catch (Exception e) {
            e.printStackTrace();
//            log.error("异常：", e);
        }
        return Result.OK();
    }


    private @NotNull OrderResponse getOrderResponse(PayOrder payOrder, PayMerchantAppKey payMerchantAppKey) {
        OrderResponse orderResponse = OrderResponse.builder().orderNo(payOrder.getOrderNo()).amount(payOrder.getAmount()).status(payOrder.getStatus()).currentTime(System.currentTimeMillis()).build();
        orderResponse.setSign(orderResponseSign(orderResponse, payMerchantAppKey));
        return orderResponse;
    }


    /**
     * 回调签名
     *
     * @param payOrder
     * @param payMerchantAppKey
     */
    private String orderResponseSign(OrderResponse payOrder, PayMerchantAppKey payMerchantAppKey) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("orderNo", payOrder.getOrderNo());
        treeMap.put("amount", payOrder.getAmount().toString());
        treeMap.put("status", payOrder.getStatus().toString());
        treeMap.put("currentTime", String.valueOf(payOrder.getCurrentTime()));
        return SignUtil.getSign(payMerchantAppKey.getAppKey(), treeMap);
    }
}