package pay.platform.api.servcie.impl;


import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.row.Db;
import com.mybatisflex.spring.service.impl.ServiceImpl;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import pay.platform.api.servcie.PayDailyTotalService;
import pay.platform.domain.PayDailyTotal;
import pay.platform.domain.PayOrder;
import pay.platform.domain.table.PayDailyTotalTableDef;
import pay.platform.mapper.PayDailyTotalMapper;

import java.util.Calendar;

/**
 * 支付订单日流水 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayDailyTotalServiceImpl extends ServiceImpl<PayDailyTotalMapper, PayDailyTotal> implements PayDailyTotalService {

    private static String ORDER_TAG = "PayDailyTotal";
    private static String ORDER_WEEK_TAG = "PayWeekTotal";

    public PayDailyTotal totalDailyPayAmount(String channelId) {
        var calendar = Calendar.getInstance();

        QueryWrapper queryWrapper = QueryWrapper.create().from(PayDailyTotalTableDef.PAY_DAILY_TOTAL).where(PayDailyTotalTableDef.PAY_DAILY_TOTAL.CHANNEL_ID.eq(channelId))
                .and(PayDailyTotalTableDef.PAY_DAILY_TOTAL.TOTAL_TIME.eq(DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd")));

        return mapper.selectOneByQuery(queryWrapper);
    }

    @Override
    @Async
    public void addDailyTotal(PayOrder payOrder) {
        var calendar = Calendar.getInstance();
        var day = DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd");
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayDailyTotalTableDef.PAY_DAILY_TOTAL)
                .where(PayDailyTotalTableDef.PAY_DAILY_TOTAL.CHANNEL_ID.eq(payOrder.getChannelId()))
                .and(PayDailyTotalTableDef.PAY_DAILY_TOTAL.TOTAL_TIME.eq(day));

        var payDailyTotal = mapper.selectOneByQuery(queryWrapper);
        //统计当天已支付金额
        if (payDailyTotal == null) {
            synchronized (ORDER_TAG) {
                payDailyTotal = mapper.selectOneByQuery(queryWrapper);
                if (payDailyTotal == null) {
                    var payDailyTotal1 = PayDailyTotal.builder().totalTime(day)
//                            .agentId(payOrder.getAgentId())
//                            .merchantId(payOrder.getMerchantId())
                            .channelId(payOrder.getChannelId())
                            .amount(payOrder.getAmount()).build();
                    mapper.insert(payDailyTotal1);
                    //更新通道当天已支付金额
                    Db.updateBySql("update t_pay_daily_total set amount = ? where channel_id=? and total_time =? ", payOrder.getAmount(), payOrder.getChannelId(), day);

                    Db.updateBySql("update t_pay_channel set day_pay_amount = ?,total_pay_amount=total_pay_amount+? where id=?", payOrder.getAmount(), payOrder.getAmount(), payOrder.getChannelId());

                    return;
                }
                updatePayAmount(payOrder, day);
            }
            return;
        }
        updatePayAmount(payOrder, day);
    }

    @Override
    @Async
    public void addWeekTotal(PayOrder payOrder) {
        var calendar = Calendar.getInstance();
        var week = String.valueOf(calendar.get(Calendar.WEEK_OF_YEAR));
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayDailyTotalTableDef.PAY_DAILY_TOTAL)
                .where(PayDailyTotalTableDef.PAY_DAILY_TOTAL.CHANNEL_ID.eq(payOrder.getChannelId()))
                .and(PayDailyTotalTableDef.PAY_DAILY_TOTAL.TOTAL_TIME.eq(week));

        var payDailyTotal = mapper.selectOneByQuery(queryWrapper);
        //统计当天已支付金额
        if (payDailyTotal == null) {
            synchronized (ORDER_WEEK_TAG) {
                payDailyTotal = mapper.selectOneByQuery(queryWrapper);
                if (payDailyTotal == null) {
                    var payDailyTotal1 = PayDailyTotal.builder().totalTime(week)
//                            .agentId(payOrder.getAgentId())
//                            .merchantId(payOrder.getMerchantId())
                            .channelId(payOrder.getChannelId())
                            .amount(payOrder.getAmount()).build();
                    mapper.insert(payDailyTotal1);
                    //重置周支付金额
                    Db.updateBySql("update t_pay_daily_total set amount = ? where channel_id=? and total_time =? ", payOrder.getAmount(), payOrder.getChannelId(), week);

                    return;
                }

                Db.updateBySql("update t_pay_daily_total set amount = amount+? where channel_id=? and total_time =? ", payOrder.getAmount(), payOrder.getChannelId(), week);
            }
            return;
        }
        Db.updateBySql("update t_pay_daily_total set amount = amount+? where channel_id=? and total_time =? ", payOrder.getAmount(), payOrder.getChannelId(), week);
    }

    private static void updatePayAmount(PayOrder payOrder, String day) {
        Db.updateBySql("update t_pay_channel set day_pay_amount = day_pay_amount+?,total_pay_amount=total_pay_amount+? where id=?", payOrder.getAmount(), payOrder.getAmount(), payOrder.getChannelId());
        Db.updateBySql("update t_pay_daily_total set amount = amount+? where channel_id=? and total_time =? ", payOrder.getAmount(), payOrder.getChannelId(), day);
    }
}